/*-
 * Copyright (C)2021 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: qrben.h,v 1.3 2021/08/07 10:17:13 babolo Exp $
 */

#define QRBEN_O(IF, T)        do{if(!!(IF)) {ifBLIN_QW0 T; goto out;};}while(0)
#define QRBEN_W(IF, T, EX)    do{if(!!(IF)) {ifBLIN_QW0 T; ex = (EX); goto out;};}while(0)
#define QRBEN_X(IF, T, E, EX) do{if(!!(IF)) {ifBLIN_QX0 T; ex = (EX); errno = (E); goto out;};}while(0)

typedef struct qrcodegen_Segment qrben_seg;
typedef enum qrcodegen_Mode      qrben_mode;

typedef struct qrben_code {
    u_int16_t  size;
    uint8_t    code[];
} qrben_code;

typedef struct qrben_segg {
    union {
        struct {
            u_int16_t start;
            u_int16_t qlas;
        };
        u_int32_t     ecinum;
    };
} qrben_segg;

typedef struct qrben_segsp {
    u_int      spn;
    qrben_segg sp[];
} qrben_segsp;

typedef struct qrben_conf {
    BLIN_flag    flags;
#   define     QRBEN_FIXSIZE 0x000100       /*   QR            */
#   define     QRBEN_FIXERRC 0x000200       /*   QR         */
#   define     QRBEN_SHOW    0x000400       /*   stdout                                        */
#   define     QRBEN_BORDER  0x000800       /*  -i                                     */
    int          nseg;                 /*                                  */
    qrben_seg   *segs;                 /*                                             */
    char        *imgname;              /*                                */
    int          smax;                 /*                                 */
    uint8_t      versize;              /*   QR  (1..40)                             */
    uint8_t      errc;                 /*    (0..3)                            */
    uint8_t      x;                    /*                          */
    uint8_t      y;                    /*                            */
} qrben_conf;

extern const char    qrben_num[];
extern const char    qrben_alpha[];
extern const uint8_t qrben_qlass[128];

int         qrben_segsplen __P((qrben_conf *cf, qrben_segsp *segs, u_int versize));
int         qrben_pseg     __P((qrben_conf *cf, u_int16_t len, const char *text, u_int versize));
int         qrben_aseg     __P((qrben_conf *cf, u_int16_t len, const char *text, u_int versize));
int         qrben_addseg   __P((qrben_conf *cf, u_int16_t len, const char *text, qrben_mode mode));
int         qrben_addeci   __P((qrben_conf *cf, u_int num));
int         qrben_pbm      __P((qrben_conf *cf, qrben_code *qr, FILE *img));
int         qrben_pbmsz    __P((qrben_conf *cf, qrben_code *qr, FILE *img));
int         qrben_show     __P((qrben_conf *cf, qrben_code *qr));
qrben_code *qrben_gen      __P((qrben_conf *cf));
